/*
	Utility to open a Diapason session html file and change the organization of the steps, then save these changes
	See: https://direns.mines-paristech.fr/Sites/Thopt/en/co/module_Logiciel_2.html
	
	Copyright Renaud Gicquel 2023
	This document is licensed under CC BY-SA 4.0 (Attribution-ShareAlike 4.0 International)
	
*/

package rg.diap;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;


/*
 * Creates or edits a step block
 * 
 */

public class CreateStep extends JFrame implements ActionListener{
	
	static ResourceBundle res = ResourceBundle.getBundle("rg.diap.Res");
	  JLabel jLabelTitle = new JLabel();
	  JTextField titleField = new JTextField();
	  JLabel jLabelImageFile = new JLabel();
	  JLabel jLabelStep = new JLabel();
	  JTextField imageField = new JTextField();
	  JLabel jLabeSoundFile = new JLabel();
	  JTextField soundField = new JTextField();
	  JTextField champAudio = new JTextField();
	  JButton bImageBrowse = new JButton();
	  JButton bSoundBrowse = new JButton();
	  HTML_GUI html;
	  
	  JButton bValidateHtml = new JButton();
	  JLabel jLabel6 = new JLabel();
	  JLabel jLabelAudioId = new JLabel();
	  JLabel jLabelURL = new JLabel();
	  JLabel jLabelLabel = new JLabel();
	  JTextField labelField = new JTextField();
	  JTextField urlField = new JTextField();
	  JButton bURL = new JButton();
	  JButton bAddLink = new JButton();
	  Mp3Filter mp3Filter=new Mp3Filter();
	  JpegSwfFilter jpegFilter = new JpegSwfFilter();
	  LinkFilter linkFilter = new LinkFilter();
	  JLabel labelTitre3 = new JLabel();
	  JScrollPane jScrollPane3 = new JScrollPane();
	  JTextArea stepBlock = new JTextArea();
      JScrollPane scrollPane = new JScrollPane(stepBlock);
  
	  String linkDir="";
	  Element element;
	  int selectedIndex;
	  HyperlinkLabel hyperlinkLabel = new HyperlinkLabel(res.getString("check_syntax"),"https://validator.nu/#textarea");
	  
	  public CreateStep(HTML_GUI html) {
		  super();
		  this.html=html;
	  }
	  
	  public CreateStep() {
	    try {
	      jbInit();
	      setVisible(true);
	      setTitle(res.getString("Diapason_step_editor"));

	    }
	    catch(Exception e) {
	      e.printStackTrace();
	    }
	  }
	  private void jbInit() throws Exception {
	    setSize(new Dimension(869, 750));
	    this.getContentPane().setLayout(null);
	    jLabelTitle.setText(res.getString("Step_title"));
	    jLabelTitle.setBounds(new Rectangle(3, 6, 150, 24));
	    titleField.setText("");
	    titleField.setBounds(new Rectangle(164, 6, 535, 21));
	    jLabelImageFile.setText(res.getString("Image"));
	    jLabelImageFile.setBounds(new Rectangle(3, 42, 141, 18));
	    imageField.setText("");
	    imageField.setBounds(new Rectangle(164, 42, 259, 19));
	    jLabelAudioId.setText("Audio id");
	    jLabelAudioId.setBounds(new Rectangle(600, 70, 100, 18));
	    champAudio.setText("");
	    champAudio.setBounds(new Rectangle(710, 70, 100, 20));
	    jLabeSoundFile.setText(res.getString("Sound"));
	    jLabeSoundFile.setBounds(new Rectangle(3, 70, 153, 15));
	    soundField.setText("");
	    soundField.setBounds(new Rectangle(165, 70, 259, 20));
	    jLabelStep.setText("");
	    jLabelStep.setBounds(new Rectangle(600, 42, 200, 20));
	    
	    
	    bImageBrowse.setBounds(new Rectangle(443, 35, 117, 27));
	    bImageBrowse.setText(res.getString("Browse"));
	    bImageBrowse.addActionListener(new CreateStep_bImageBrowse_actionAdapter(this));
	    bSoundBrowse.setBounds(new Rectangle(443, 68, 119, 24));
	    bSoundBrowse.setText(res.getString("Browse"));
	    bSoundBrowse.addActionListener(new CreateStep_bSoundBrowse_actionAdapter(this));
	    bValidateHtml.setBounds(new Rectangle(647, 175, 149, 37));
	    bValidateHtml.setFont(new java.awt.Font("Dialog", 5, 17));

	    bValidateHtml.setText(res.getString("Validate_HTML"));
	    bValidateHtml.addActionListener(new CreateStep_bValidateHtml_actionAdapter(this));

	    hyperlinkLabel.setBounds(new Rectangle(5, 180, 300, 35));
	    hyperlinkLabel.setFont(new java.awt.Font("Dialog", 5, 17));
	    stepBlock.setBounds(new Rectangle(5, 220, 790, 440));
	    scrollPane.setBounds(new Rectangle(5, 220, 800, 450));
	    stepBlock.setLineWrap(true);        
	    stepBlock.setWrapStyleWord(true);   
	    
	    jLabel6.setBounds(new Rectangle(3, 200, 150, 24));
	    jLabel6.setText("");
	    jLabelLabel.setBounds(new Rectangle(5, 140, 50, 15));
	    jLabelLabel.setText(res.getString("Label"));
	    labelField.setBounds(new Rectangle(60, 140, 450, 20));
	    labelField.setText("");
	    jLabelURL.setBounds(new Rectangle(3, 120, 50, 15));
	    jLabelURL.setText("URL");
	    urlField.setBounds(new Rectangle(60, 120, 450, 20));
	    urlField.setText("");
	    bURL.setBounds(new Rectangle(515, 125, 120, 26));
	    bURL.setText(res.getString("Browse"));
	    bURL.addActionListener(new CreateStep_bURL_actionAdapter(this));
	    bAddLink.setBounds(new Rectangle(647, 125, 120, 26));
	    bAddLink.setText(res.getString("Add_link"));
	    bAddLink.addActionListener(new CreateStep_bAddLink_actionAdapter(this));
	    labelTitre3.setBounds(new Rectangle(0, 0, 221, 30));
	    labelTitre3.setText(res.getString("Title"));
	    labelTitre3.setFont(new java.awt.Font("Dialog", 0, 15));

	    this.getContentPane().add(jLabelTitle, null);
	    this.getContentPane().add(hyperlinkLabel, null);
	    
	    this.getContentPane().add(titleField, null);
	    this.getContentPane().add(imageField, null);
	    this.getContentPane().add(jLabelAudioId, null);
	    this.getContentPane().add(champAudio, null);
	    this.getContentPane().add(jLabelStep, null);		    
	    this.getContentPane().add(jLabelImageFile, null);
	    this.getContentPane().add(jLabeSoundFile, null);
	    this.getContentPane().add(soundField, null);
	    this.getContentPane().add(bImageBrowse, null);
	    this.getContentPane().add(bSoundBrowse, null);
	    this.getContentPane().add(bURL, null);
	    this.getContentPane().add(jLabel6, null);
	    this.getContentPane().add(scrollPane, null);
	    this.getContentPane().add(bAddLink, null);
	    this.getContentPane().add(bValidateHtml, null);
	    this.getContentPane().add(urlField, null);
	    this.getContentPane().add(labelField, null);
	    this.getContentPane().add(jLabelURL, null);
	    this.getContentPane().add(jLabelLabel, null);
	    this.getContentPane().add(jScrollPane3, null);
	    bValidateHtml.addActionListener(this);
	  }
	  
	  public void actionPerformed(ActionEvent e) {
	  }

	  void bImageBrowse_actionPerformed(ActionEvent e) {
			  String lastOpenedDir=html.lastOpenedFile.getName();
			  lastOpenedDir=lastOpenedDir.substring(0,lastOpenedDir.length()-5);
		    String dir="distrib"+File.separator+lastOpenedDir;

		    File fich=new File(dir);
		    JFileChooser fileChooser=new JFileChooser();
		    if(fich.exists()) fileChooser.setCurrentDirectory(fich);
		      else{
		        fileChooser.setCurrentDirectory(new File(lastOpenedDir));
		        String msg = res.getString("Beware_Folder")+lastOpenedDir
		        		+res.getString("does_not_exist");
		        JOptionPane.showMessageDialog(this, msg);

		      }
		    fileChooser.setFileFilter(jpegFilter);
		    fileChooser.showOpenDialog(null);
		    if(fileChooser.getSelectedFile()!=null){
		      String filePath = fileChooser.getSelectedFile().getName();
		      imageField.setText(lastOpenedDir+"/"+getFileReference(filePath));
	           Element td = element.select("td").first();
	           Element img = td.select("img").first();
	           img.attr("src", imageField.getText());
				  stepBlock.setText(htmlFormat(element.toString()));

		    }

		  }

	  void bSoundBrowse_actionPerformed(ActionEvent e) {
			  String lastOpenedDir=html.lastOpenedFile.getName();
			  lastOpenedDir=lastOpenedDir.substring(0,lastOpenedDir.length()-5);
		    String dir="distrib"+File.separator+lastOpenedDir;

		    File fich=new File(dir);
		    JFileChooser fileChooser=new JFileChooser();
		    if(fich.exists()) fileChooser.setCurrentDirectory(fich);
		      else{
		        fileChooser.setCurrentDirectory(new File(lastOpenedDir));
		        String msg = res.getString("Beware_Folder")
		        		+lastOpenedDir+res.getString("does_not_exist");
		        JOptionPane.showMessageDialog(this, msg);

		      }
		    fileChooser.setFileFilter(mp3Filter);
		    fileChooser.showOpenDialog(null);
		    if(fileChooser.getSelectedFile()!=null){
		      String filePath = fileChooser.getSelectedFile().getName();
		      soundField.setText(lastOpenedDir+"/"+getFileReference(filePath));
	           Element td = element.select("td").first();
	           Element source = td.select("source").first();
	           source.attr("src", soundField.getText());
				  stepBlock.setText(htmlFormat(element.toString()));
		    }
		  }

	  void bValidateHtml_actionPerformed(ActionEvent e) {
		  	//We retrieve the contents of the editing window
            Document document = Jsoup.parse(stepBlock.getText(), "UTF-8");
            //Warning: Jsoup has completed the code to make it a valid html page
           
            //we create the two Element of the tr and td tags
            Element tr = new Element("tr").attr("style", "display: none;");
            Element td = new Element("td");
            
            //We retrieve the contents of the body tag
            Element body = document.selectFirst("body"); 
            Elements blocItems = body.children();
            Element audio = document.selectFirst("audio");

            //If audio code is missing, create one by concatenation
            String audioCode="";
            String codeAudio="audio"+selectedIndex;
            if(audio!=null){;
            	audioCode= audio.attr("id");
            	if(audioCode.equals(""))audio.attr("id",codeAudio);
            }
            
            // on ajoute  <td> le contenu de <body>
            for (Element item : blocItems) {
                td.appendChild(item);
  		    }
            // We place <td> in <tr>
            tr.appendChild(td);//tr contient le nouveau bloc
            
         
            //Validate the session in the Vector vSlides
             html.updateStepBlock(selectedIndex,tr);
             stepBlock.setText(htmlFormat(tr.toString()));

	  }
	  
	  String htmlFormat(String source){
		  String result="";
		  result=source.replaceAll("</br>", "\n</br>");
		  result=result.replaceAll("</td>", "\n</td>");
		  result=result.replaceAll("<a ", "\n<a ");
		  
		  return result;		  
	  }       
 		  
	  void populate(){
		  if(element!=null)populate(element, selectedIndex);
	  }		  
	  
	  void populate(Element el, int selectedIndex){
		  this.selectedIndex=selectedIndex;
		  
		  jLabelStep.setText("Step n"+ selectedIndex);

          element= el.clone();
          Element h2 = element.select("h2").first();
          titleField.setText(h2.text());
          
          Element img = element.select("img").first();
          if(img!=null)imageField.setText(img.attr("src"));
          Element audio = element.select("audio").first();
          Element source=null;
          if(audio!=null)source = audio.select("source").first();
          
          if(source!=null)soundField.setText(source.attr("src"));
          if(audio!=null)champAudio.setText(audio.attr("id"));

          stepBlock.setText(htmlFormat(element.toString()));
		  
	  }

	  void _actionPerformed(ActionEvent e) {

		  }

	  void bURL_actionPerformed(ActionEvent e) {
		  String lastOpenedDir=html.lastOpenedFile.getName();
		  lastOpenedDir=lastOpenedDir.substring(0,lastOpenedDir.length()-5);
	    String dir="distrib"+File.separator+lastOpenedDir;

	    File fich=new File(dir);
	    JFileChooser fileChooser=new JFileChooser();
	    if(fich.exists()) fileChooser.setCurrentDirectory(fich);
	      else{
	        fileChooser.setCurrentDirectory(new File(lastOpenedDir));
	        String msg = res.getString("Beware_Folder")
	        		+lastOpenedDir+res.getString("does_not_exist");
	        JOptionPane.showMessageDialog(this, msg);

	      }
	    fileChooser.setFileFilter(linkFilter);
		    fileChooser.showOpenDialog(null);
		    if(fileChooser.getSelectedFile()!=null){
		      String filePath=fileChooser.getSelectedFile().getName();
		      urlField.setText(lastOpenedDir + "/" + filePath);
		    }

		  }

	  String getFileReference(String filePath){
	    return filePath;
	  }

	  void bAddLink_actionPerformed(ActionEvent e) {
		  String label= labelField.getText();
		  String URL= urlField.getText();
		  	Element link = new Element("a");
		  	link.attr("href", URL);
		  	link.attr("target", "_blank");
		  	link.text(label);
            Element td = element.select("td").first();

		  if((!label.equals(""))&&(!URL.equals(""))){
			  td.appendChild(link);
			  stepBlock.setText(htmlFormat(element.toString()));
		  }
	 }

}
	class Mp3Filter extends javax.swing.filechooser.FileFilter{
		  static ResourceBundle res = ResourceBundle.getBundle("rg.diap.Res");
		  public boolean accept(File f){
		    boolean accept=f.isDirectory();
		    if(!accept){
		      String suffix=getSuffix(f);
		      if(suffix!=null)accept=suffix.equals("mp3");
		    }
		    return accept;
		  }
		    public String getDescription(){
		      return res.getString("mp3_Files");
		    }
		    public String getSuffix(File f){
		      String s = f.getPath(),suffix=null;
		      int i=s.lastIndexOf('.');
		      if(i>0 && i<s.length()-1)suffix = s.substring(i+1).toLowerCase();
		        return suffix;
		    }
	}
	  class JpegSwfFilter extends javax.swing.filechooser.FileFilter{
		  static ResourceBundle res = ResourceBundle.getBundle("rg.diap.Res");
		   public boolean accept(File f){
		     boolean accept=f.isDirectory();
		     if(!accept){
		       String suffix=getSuffix(f);
		       if (suffix != null) accept = suffix.equals("jpg") || suffix.equals("png");
		     }
		     return accept;
		   }
		     public String getDescription(){
		       return res.getString("jpg_png_files");
		     }
		     public String getSuffix(File f){
		       String s = f.getPath(),suffix=null;
		       int i=s.lastIndexOf('.');
		       if(i>0 && i<s.length()-1)suffix = s.substring(i+1).toLowerCase();
		         return suffix;
		     }
		 }
	  
	  class LinkFilter extends javax.swing.filechooser.FileFilter{
		  static ResourceBundle res = ResourceBundle.getBundle("rg.diap.Res");
		   public boolean accept(File f){
		     boolean accept=f.isDirectory();
		     if(!accept){
		       String suffix=getSuffix(f);
		       if (suffix != null) accept = !suffix.equals("jpg") && !suffix.equals("png") && !suffix.equals("mp3");
		     }
		     return accept;
		   }
		     public String getDescription(){
		       return res.getString("files_other_mp3_jpg_png");
		     }
		     public String getSuffix(File f){
		       String s = f.getPath(),suffix=null;
		       int i=s.lastIndexOf('.');
		       if(i>0 && i<s.length()-1)suffix = s.substring(i+1).toLowerCase();
		         return suffix;
		     }
		 }


		class CreateStep_bImageBrowse_actionAdapter implements java.awt.event.ActionListener {
		  CreateStep adaptee;

		  CreateStep_bImageBrowse_actionAdapter(CreateStep adaptee) {
		    this.adaptee = adaptee;
		  }
		  public void actionPerformed(ActionEvent e) {
		    adaptee.bImageBrowse_actionPerformed(e);
		  }
		}

		class CreateStep_bSoundBrowse_actionAdapter implements java.awt.event.ActionListener {
		  CreateStep adaptee;

		  CreateStep_bSoundBrowse_actionAdapter(CreateStep adaptee) {
		    this.adaptee = adaptee;
		  }
		  public void actionPerformed(ActionEvent e) {
		    adaptee.bSoundBrowse_actionPerformed(e);
		  }
		}

		class CreateStep_bValidateHtml_actionAdapter implements java.awt.event.ActionListener {
		  CreateStep adaptee;

		  CreateStep_bValidateHtml_actionAdapter(CreateStep adaptee) {
		    this.adaptee = adaptee;
		  }
		  public void actionPerformed(ActionEvent e) {
		    adaptee.bValidateHtml_actionPerformed(e);
		  }
		}


		class CreateStep_bURL_actionAdapter implements java.awt.event.ActionListener {
		  CreateStep adaptee;

		  CreateStep_bURL_actionAdapter(CreateStep adaptee) {
		    this.adaptee = adaptee;
		  }
		  public void actionPerformed(ActionEvent e) {
		    adaptee.bURL_actionPerformed(e);
		  }
		}

		class CreateStep_bAddLink_actionAdapter implements java.awt.event.ActionListener {
		  CreateStep adaptee;

		  CreateStep_bAddLink_actionAdapter(CreateStep adaptee) {
		    this.adaptee = adaptee;
		  }
		  public void actionPerformed(ActionEvent e) {
		    adaptee.bAddLink_actionPerformed(e);
		  }

		}


			