/*
	Utility to open a Diapason session html file and change the organization of the steps, then save these changes
	See: https://direns.mines-paristech.fr/Sites/Thopt/en/co/module_Logiciel_2.html
	
	Copyright Renaud Gicquel 2023
	This document is licensed under CC BY-SA 4.0 (Attribution-ShareAlike 4.0 International)
	
*/

package rg.diap;

import java.awt.Color;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.JLabel;

import java.awt.*;
import java.awt.event.*;
import java.io.FileWriter;
import java.io.IOException;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

/*
 * Overall editor of the html file
 * 
 */


public class HTML_GUI  extends JFrame {
	static ResourceBundle res = ResourceBundle.getBundle("rg.diap.Res");
    private File currentFile; //Variable to store the currently open HTML file
    File lastOpenedFile; // Variable to store the last opened file
    private JFrame frame;
    private JList<String> slideList;
    private DefaultListModel<String> listModel;
    private JScrollPane scrollPane;
    private Vector<Element> vSlide= new Vector();
    CreateStep step ;

    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu(res.getString("File"));
    JMenuItem openItem = new JMenuItem(res.getString("Open_HTML_file"));
    JMenuItem saveItem = new JMenuItem(res.getString("Save"));
    JMenuItem quitMenuItem = new JMenuItem(res.getString("Quit"));
    JMenu aboutMenu = new JMenu(res.getString("About"));
    JMenuItem aboutMenuItem = new JMenuItem(res.getString("About_Diapason"));
    JButton upButton, downButton, insertButton, deleteButton,
    changeTitleButton, duplicateButton;
    JLabel lblDiapason, fileLabel;
	/**
	 * Create the application.
	 */
	public HTML_GUI() {
		initialize();
	}
	JPanel contentPane ;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					HTML_GUI window = new HTML_GUI();
					window.frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
	    listModel = new DefaultListModel<>();
	    slideList = new JList<>(listModel);

		setBounds(10, 10, 850, 700);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		contentPane = new JPanel();
		contentPane.setForeground(Color.RED);
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
	    frame = new JFrame(res.getString("HTML_Viewer"));
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    frame.setSize(600, 600);
	    
    	step=new CreateStep();
    	step.html=this;

        slideList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION); // Limits the selection to a single row

        slideList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    int index = slideList.locationToIndex(e.getPoint());
                    if (index >= 0) {
                        step.setVisible(true);
                        step.populate(vSlide.elementAt(index),index);
                    }
                }
            }
        });

	    contentPane.setLayout(null);

	    scrollPane = new JScrollPane(slideList);
	    scrollPane.setBounds(0, 50, 400, 450);
	    contentPane.add(scrollPane);
	    
	    upButton = new JButton("^");
	    upButton.setFont(new Font("Tahoma", Font.PLAIN, 18));
	    upButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
                moveSelectedItemUp();
	    	}
	    });
	    upButton.setBounds(444, 81, 97, 39);
	    contentPane.add(upButton);
	    
	    downButton = new JButton("v");
	    downButton.setFont(new Font("Tahoma", Font.PLAIN, 18));
	    downButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		 moveSelectedItemDown();
	    	}
	    });
	    downButton.setBounds(444, 133, 97, 38);
	    contentPane.add(downButton);
	    
	    insertButton = new JButton(res.getString("Insert"));
	    insertButton.setFont(new Font("Tahoma", Font.PLAIN, 18));
	    insertButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		insertNewItem();
	    	}
	    });
	    insertButton.setBounds(444, 184, 97, 43);
	    contentPane.add(insertButton);
	    
	    deleteButton = new JButton(res.getString("Delete"));
	    deleteButton.setFont(new Font("Tahoma", Font.PLAIN, 18));
	    deleteButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		deleteItem();
	    	}
	    });
	    deleteButton.setBounds(422, 235, 141, 43);
	    contentPane.add(deleteButton);

	    changeTitleButton = new JButton(res.getString("Change_Title"));
	    changeTitleButton.setFont(new Font("Tahoma", Font.PLAIN, 18));
	    changeTitleButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		renameItem();
	    	}
	    });
	    changeTitleButton.setBounds(412, 286, 160, 43);
	    contentPane.add(changeTitleButton);

	    duplicateButton = new JButton(res.getString("Duplicate"));
	    duplicateButton.setFont(new Font("Tahoma", Font.PLAIN, 18));
	    duplicateButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		duplicateItem();
	    	}
	    });
	    duplicateButton.setBounds(412, 337, 160, 43);
	    contentPane.add(duplicateButton);

	    lblDiapason = new JLabel(res.getString("Diapason_file_editor"));
	    lblDiapason.setForeground(Color.BLUE);
	    lblDiapason.setFont(new Font("Tahoma", Font.PLAIN, 18));
	    lblDiapason.setBounds(10, 0, 241, 50);
	    frame.getContentPane().add(contentPane);
	    
	    fileLabel = new JLabel(res.getString("File_:"));
	    fileLabel.setForeground(Color.BLUE);
	    fileLabel.setFont(new Font("Tahoma", Font.PLAIN, 14));
	    fileLabel.setBounds(60, 0, 660, 50);
	    contentPane.add(fileLabel);
	    	    
	    
	    fileMenu.addSeparator();
	    fileMenu.add(quitMenuItem);
	    fileMenu.add(openItem);
	    fileMenu.add(saveItem);
	    aboutMenu.add(aboutMenuItem);
	    menuBar.add(fileMenu);
	    menuBar.add(aboutMenu);
	    frame.setJMenuBar(menuBar);
	    
	    frame.setVisible(true);

	    openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK));
	    openItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	            JFileChooser fileChooser = new JFileChooser(new File("existing"));
	            fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	            int result = fileChooser.showOpenDialog(frame);
	            if (result == JFileChooser.APPROVE_OPTION) {
	                File selectedFile = fileChooser.getSelectedFile();
	                loadHTMLFile(selectedFile);
	                lastOpenedFile = selectedFile; // Mettre  jour le dernier fichier ouvert
	                try {
						fileLabel.setText(res.getString("File_:")+lastOpenedFile.getCanonicalPath());
					} catch (IOException e1) {
						e1.printStackTrace();
					}
	            }
	        }
	    });
	    saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK));
	    saveItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	            saveHTMLFile();
	        }
	    });

	    quitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
	    quitMenuItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	            System.exit(0);
	        }
	    });

	    aboutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_DOWN_MASK));
	    aboutMenuItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	            displayAbout();
	        }
	    });

	    
	}
	
    private void displayAbout() {
    	JOptionPane.showMessageDialog(null, 
    			res.getString("About_CC")
    			, res.getString("About"), JOptionPane.INFORMATION_MESSAGE);	    
    }
	    
	   private void moveSelectedItemUp() {
	        int selectedIndex = slideList.getSelectedIndex();
	        if (selectedIndex > 0) {
	            String selectedValue = listModel.get(selectedIndex);
	            listModel.remove(selectedIndex);
	            listModel.add(selectedIndex - 1, selectedValue);
	            slideList.setSelectedIndex(selectedIndex - 1);
	            Element el= vSlide.elementAt(selectedIndex);
	            vSlide.removeElementAt(selectedIndex);
	            vSlide.add(selectedIndex - 1, el);
	        }
	    }
	    
	    
	    private void moveSelectedItemDown() {
	        int selectedIndex = slideList.getSelectedIndex();
	        if (selectedIndex >= 0 && selectedIndex < listModel.size() - 1) {
	            String selectedValue = listModel.get(selectedIndex);
	            listModel.remove(selectedIndex);
	            listModel.add(selectedIndex + 1, selectedValue);
	            slideList.setSelectedIndex(selectedIndex + 1);
	            Element el= vSlide.elementAt(selectedIndex);
	            vSlide.removeElementAt(selectedIndex);
	            vSlide.add(selectedIndex + 1, el);

	        }
	    }
	    
	    private void insertNewItem() {
	        int selectedIndex = slideList.getSelectedIndex();
	        if (selectedIndex >= 0) {
	            String newItemName = JOptionPane.showInputDialog(frame, res.getString("Enter_step_name"));
	            if (newItemName != null && !newItemName.isEmpty()) {
	                listModel.add(selectedIndex, newItemName);
	                Element tr = new Element("tr").attr("style", "display: none;");
	                Element td = new Element("td");
	                Element h2 = new Element("h2").text(newItemName);
	                Element p = new Element("p").text("Insert here your new page");
	                td.appendChild(h2);
	                td.appendChild(p);
	                tr.appendChild(td);
	                vSlide.add(selectedIndex, tr);
	                slideList.setSelectedIndex(selectedIndex);

	            }
	        } else {//Put it at the end if there is no one selected
	            String newItemName = JOptionPane.showInputDialog(frame, res.getString("Enter_step_name"));
	            if (newItemName != null && !newItemName.isEmpty()) {
	                listModel.addElement(newItemName);
	                Element tr = new Element("tr").attr("style", "display: none;");
	                Element td = new Element("td");
	                Element h2 = new Element("h2").text(newItemName);
	                Element p = new Element("p").text(res.getString("Insert_new_step"));
	                td.appendChild(h2);
	                td.appendChild(p);
	                tr.appendChild(td);
	                vSlide.add(tr);
	                slideList.setSelectedIndex(vSlide.size()-1);
	           }
	        }
	    }
	    
    private void renameItem() {
        int selectedIndex = slideList.getSelectedIndex();
        String oldName=listModel.get(selectedIndex);
        if (selectedIndex >= 0) {
        	String newItemName = JOptionPane.showInputDialog(frame, res.getString("Enter_new_title"),oldName);
            if (newItemName != null && !newItemName.isEmpty()) {
                listModel.setElementAt(newItemName, selectedIndex); 
                Element element = vSlide.elementAt(selectedIndex);
                Element h2 = element.select("h2").first();
                Element img = element.select("img").first();
                Element audio = element.select("audio").first();
                Element source = audio.select("source").first();
                
              	h2.text(newItemName);

                Element tr = new Element("tr").attr("style", "display: none;");
                Element td = new Element("td");
                td.appendChild(h2);
                td.appendChild(img);
                audio.appendChild(source);
                td.appendChild(audio);
                tr.appendChild(td);
              
                // Check if the item has been found
                if (h2 != null) {
     	            vSlide.removeElementAt(selectedIndex);
    	            vSlide.add(selectedIndex, tr);
                 }
            }
        }
    }
    
    private void duplicateItem() {
	        int selectedIndex = slideList.getSelectedIndex();
	        String oldName=listModel.get(selectedIndex);
	        if (selectedIndex >= 0) {
	            int choice = JOptionPane.showConfirmDialog(frame, 
	            		res.getString("Please_confirm_duplicate_step"), res.getString("Yes"), JOptionPane.OK_CANCEL_OPTION);
	            if (choice == JOptionPane.OK_OPTION) {
		            String newItemName = JOptionPane.showInputDialog(frame, res.getString("Enter_name_new_step"),oldName);
		            if (newItemName != null && !newItemName.isEmpty()) {
		                listModel.add(selectedIndex + 1, newItemName);
		                
		                Element el = vSlide.elementAt(selectedIndex);

	                    Element element= el.clone();
	                    Element h2 = element.select("h2").first();
	                    Element img = element.select("img").first();
	                    Element audio = element.select("audio").first();
	                    Element source = audio.select("source").first();
	                    
	                  	h2.text(newItemName);

		                Element tr = new Element("tr").attr("style", "display: none;");
		                Element td = new Element("td");
		                td.appendChild(h2);
		                td.appendChild(img);
		                audio.appendChild(source);
		                td.appendChild(audio);
		                tr.appendChild(td);

	                    
	                    // Check if the item has been found
	                    if (h2 != null) {
	        	            vSlide.add(selectedIndex + 1, tr);
	        	            slideList.setSelectedIndex(selectedIndex + 1);
	                     }
		            }
		        }
            }
	    }
	    
	    private void deleteItem() {
	        int selectedIndex = slideList.getSelectedIndex();
	        if (selectedIndex >= 0) {
	            int choice = JOptionPane.showConfirmDialog(frame, 
	            		res.getString("Please_confirm_delete_step"), res.getString("Yes"), JOptionPane.OK_CANCEL_OPTION);
	            if (choice == JOptionPane.OK_OPTION) {
		            listModel.remove(selectedIndex);
		            vSlide.removeElementAt(selectedIndex);
	            }
	        }
	    }

	    
	    private void loadHTMLFile(File file) {
	        listModel.clear();
	        currentFile = file; 

	        try {
	            Document doc = Jsoup.parse(file, "UTF-8");
	            Element slidesContainer = doc.getElementById("slidesContainer");
	            Elements slideElements = slidesContainer.select("tr");

	            for (int i = 0; i < slideElements.size(); i++) {
	                Element slide = slideElements.get(i);
	                Elements h2 = slide.select("h2");
	                if (!h2.isEmpty()) {
	                    String title = h2.text();
	                    listModel.addElement(title);
	                }
	                vSlide.addElement(slide);
	                
	            }
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	    }
	    
	    void updateStepBlock(Integer index, Element newElement){
	    	
	    	vSlide.setElementAt(newElement, index); 
	    	
	    }

	private void saveHTMLFile() {
	    if (currentFile != null) {
	        JFileChooser fileChooser = new JFileChooser(new File("distrib"));
	        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

	        // By default, name of the last opened file
	        if (lastOpenedFile != null) {
	            String defaultFileName = lastOpenedFile.getName();
	            fileChooser.setSelectedFile(new File(defaultFileName));
	        }

	        int result = fileChooser.showSaveDialog(frame);
	        if (result == JFileChooser.APPROVE_OPTION) {
	            File selectedFile = fileChooser.getSelectedFile();

	            // Check if the selected file already exists
	            if (selectedFile.exists()) {
	                int confirmResult = JOptionPane.showConfirmDialog(
	                    frame,
	                    res.getString("File_exists_replace"),
	                    res.getString("Replace_confirmation"),
	                    JOptionPane.YES_NO_OPTION
	                );

	                // If the user chooses "No," you can exit the method
	                if (confirmResult != JOptionPane.YES_OPTION) {
	                    return;
	                }
	            }

	            try {
	                Document doc = Jsoup.parse(currentFile, "UTF-8");
	                Element slidesContainer = doc.getElementById("slidesContainer");
	                
	                // Empty the current contents of slidesContainer
	                slidesContainer.empty();

					// Create a new table item
	                Element table = new Element("table");

	                for (int i = 0; i < vSlide.size(); i++) {
	                	Element element = vSlide.elementAt(i);

	                    // Retrieve the audio element with the current ID (e.g., "audio3")
	                    Element audioElement = element.select("audio").first();

	                    // Check if the item has been found
	                    if (audioElement != null) {
	                        // Now change the ID value to the new value
	                        String value="audio"+i;
	                    	audioElement.attr("id", value);
	                    }
 	                    
	                    
	                    // Add each tr element in the table tag
	                    table.appendChild(element);
	                }

	                // Add the table tag to your slidesContainer
	                slidesContainer.appendChild(table);
	                
		            StringBuilder slideLinksHtml = new StringBuilder("");
		            Elements slideElements = slidesContainer.select("tr");

		            for (int i = 0; i < slideElements.size(); i++) {
		                Element slide = slideElements.get(i);
		                Elements h2 = slide.select("h2");
		                if (!h2.isEmpty()) {
		                    String title = h2.text();
		                    slideLinksHtml.append("\t<li onclick=\"showSlide(")
		                            .append(i)
		                            .append(")\">")
		                            .append(title)
		                            .append("</li>\n");
		                }
		            }


		            // Replaces the contents of the <ul id="slideLinks"> block with the newly generated content
		            Element slideLinksElement = doc.getElementById("slideLinks");
		            slideLinksElement.html(slideLinksHtml.toString());
	               
	                FileWriter writer = new FileWriter(selectedFile);
	                writer.write(htmlFormat(doc.outerHtml()));
	                writer.close();
	            } catch (IOException e) {
	                e.printStackTrace();
	            }
	        }
	    }
	}
	  
	  String htmlFormat(String source){
		  String result="";
		  result=source.replaceAll("</br>", "\n\t</br>");
		  result=result.replaceAll("</td>", "\n\t</td>");
		  result=result.replaceAll("<a ", "\n\t<a ");
		  
		  return result;
		  
	  }
}
